"use strict";

/* exported logger */

var Promise = require("bluebird");

var Panel = require('./Panel');
var PanelControlElementType = require('../PanelControlElementType');

var logger = require('../../logger');
var Functions = require('../../Functions');

function AbstractPanel(params) {
  Panel.call(this, params);
  var self = this;

  this._initializeGui();
  this.setOverlayDb(self.getMap().getOverlayByName(params.panelName));
  this._createEventHandlers();

  this._tabIdCount = 0;

}

AbstractPanel.prototype = Object.create(Panel.prototype);
AbstractPanel.prototype.constructor = AbstractPanel;

AbstractPanel.prototype._createEventHandlers = function() {
  var self = this;
  var searchButton = self.getControlElement(PanelControlElementType.SEARCH_BUTTON);
  var searchInput = self.getControlElement(PanelControlElementType.SEARCH_INPUT);

  var searchByQuery = function() {
    return self.searchByQuery();
  };

  searchButton.onclick = searchByQuery;
  searchInput.onkeypress = function(event) {
    if (event.keyCode === 13) {
      searchByQuery();
    }
  };

  $(searchInput).typeahead({
    minLength : 1,
  }, {
    source : function(query, callback) {
      callback(self.getAutocomplete(query));
    }
  });

  $(searchInput).on('typeahead:select', function() {
    searchByQuery();
  });

  self.getOverlayDb().addListener("onSearch", function() {
    return self.refreshSearchResults();
  });

  GuiConnector.addWindowResizeEvent(function() {
    self.onresize();
  });

  $("a[href='#" + self.getElement().id + "']").on('shown.bs.tab', function() {
    self.onresize();
  });

};

AbstractPanel.prototype._initializeGui = function() {
  var searchQueryDiv = Functions.createElement({
    type : "div",
    name : "searchQuery",
    className : "searchPanel"
  });
  this.getElement().appendChild(searchQueryDiv);
  this.setControlElement(PanelControlElementType.SEARCH_DIV, searchQueryDiv);

  var searchLabel = Functions.createElement({
    type : "div",
    name : "searchLabel",
    content : "SEARCH:"
  });
  searchQueryDiv.appendChild(searchLabel);
  this.setControlElement(PanelControlElementType.SEARCH_LABEL, searchLabel);

  var searchInputDiv = Functions.createElement({
    type : "div",
    style : "display:table"
  });
  searchQueryDiv.appendChild(searchInputDiv);
  var searchInputCell = Functions.createElement({
    type : "div",
    style : "display:table-cell"
  });
  searchInputDiv.appendChild(searchInputCell);

  var searchInput = Functions.createElement({
    type : "input",
    name : "searchInput",
    className : "input-field typeahead"
  });
  searchInputCell.appendChild(searchInput);
  this.setControlElement(PanelControlElementType.SEARCH_INPUT, searchInput);

  var searchButtonCell = Functions.createElement({
    type : "div",
    style : "display:table-cell"
  });
  searchInputDiv.appendChild(searchButtonCell);

  var searchButton = Functions.createElement({
    type : "a",
    className : "searchButton",
    content : '<img src="resources/images/icons/search.png"/>'
  });
  searchButton.href = "#";
  searchButtonCell.appendChild(searchButton);
  this.setControlElement(PanelControlElementType.SEARCH_BUTTON, searchButton);

  var searchResultsDiv = Functions.createElement({
    type : "div",
    name : "searchResults",
    className : "tabbable boxed parentTabs"
  });
  this.getElement().appendChild(searchResultsDiv);
  this.setControlElement(PanelControlElementType.SEARCH_RESULTS_DIV, searchResultsDiv);

  var searchResultsNavTabDiv = Functions.createElement({
    type : "ul",
    className : "nav nav-tabs",
    content : '<li class="active"><a href="#set1"/></li>'
  });
  searchResultsDiv.appendChild(searchResultsNavTabDiv);
  this.setControlElement(PanelControlElementType.SEARCH_RESULTS_NAV_TAB, searchResultsNavTabDiv);

  var searchResultsContentTabDiv = Functions.createElement({
    type : "div",
    className : "tab-content",
    content : '<div class="tab-pane fade active in" name="set1" id="set1"/>'
  });
  searchResultsDiv.appendChild(searchResultsContentTabDiv);
  this.setControlElement(PanelControlElementType.SEARCH_RESULTS_CONTENT_TAB, searchResultsContentTabDiv);

};

AbstractPanel.prototype.setOverlayDb = function(overlayDb) {
  if (overlayDb === undefined) {
    throw new Error("Undefined overlayDb for panel: " + this.getPanelName());
  }
  this._overlayDb = overlayDb;
};

AbstractPanel.prototype.getOverlayDb = function() {
  return this._overlayDb;
};

AbstractPanel.prototype.clearResults = function() {
  var navElement = this.getControlElement(PanelControlElementType.SEARCH_RESULTS_NAV_TAB);
  while (navElement.firstChild) {
    navElement.removeChild(navElement.firstChild);
  }

  var contentElement = this.getControlElement(PanelControlElementType.SEARCH_RESULTS_CONTENT_TAB);
  while (contentElement.firstChild) {
    contentElement.removeChild(contentElement.firstChild);
  }
};

AbstractPanel.prototype.refreshSearchResults = function() {
  var self = this;
  self.clearResults();
  var searchDb = self.getOverlayDb();
  var queries = searchDb.getQueries();

  var promises = [];
  for (var i = 0; i < queries.length; i++) {
    promises.push(searchDb.getElementsByQuery(queries[i]));
  }
  return Promise.all(promises).then(function(results) {
    for (var i = 0; i < queries.length; i++) {
      self.addResultTab(queries[i], results[i]);
    }
    self.onresize();
  });

};

AbstractPanel.prototype.getAutocomplete = function() {
  logger.warn("Get autocomplete not implemented");
};

AbstractPanel.prototype.searchByQuery = function() {
  throw new Error("searchByQuery is not implemnted");
};

AbstractPanel.prototype.addResultTab = function(query, elements) {
  var name = JSON.parse(query).query;

  var tabId = this.getPanelName() + "Tab_" + this._tabIdCount;
  this._tabIdCount++;

  var navElement = this.getControlElement(PanelControlElementType.SEARCH_RESULTS_NAV_TAB);
  var contentElement = this.getControlElement(PanelControlElementType.SEARCH_RESULTS_CONTENT_TAB);
  var navClass = '';
  var contentClass = 'tab-pane';
  if (navElement.children.length === 0) {
    navClass = "active";
    contentClass = "tab-pane active";
  }

  var navLi = document.createElement("li");
  navLi.className = navClass;

  var navLink = Functions.createElement({
    type : "a",
    href : "#" + tabId,
    onclick : function() {
      $(this).tab('show');
    }
  });
  if (name !== undefined) {
    if (name.length > 12) {
      name = name.substring(0, 10) + "...";
    }
    navLink.innerHTML = name;
  }
  navLi.appendChild(navLink);
  if (query.name !== undefined) {
    navLink.innerHTML = query.name;
  }
  navElement.appendChild(navLi);

  var contentDiv = document.createElement("div");
  contentDiv.className = "pre-scrollable " + contentClass;
  contentDiv.style.maxHeight = "10px";

  contentDiv.id = tabId;

  contentElement.appendChild(contentDiv);

  contentDiv.appendChild(this.createPreamble(elements.element));

  var tableDiv = document.createElement("table");
  tableDiv.className = "table table-bordered";
  contentDiv.appendChild(tableDiv);
  var tableBody = document.createElement("tbody");
  tableDiv.appendChild(tableBody);

  for (var i = 0; i < elements.length; i++) {
    var element = elements[i].element;
    var icon = elements[i].icon;
    tableBody.appendChild(this.createTableElement(element, icon));
  }
};

AbstractPanel.prototype.createTargetRow = function(target, icon) {
  var self = this;
  var result = document.createElement("tr");
  var iconColumn = document.createElement("td");
  iconColumn.style.width = "20px";
  iconColumn.style.verticalAlign = "middle";
  iconColumn.style.textAlign = "center";
  result.appendChild(iconColumn);
  if (target.getTargetElements().length > 0) {
    iconColumn.appendChild(self.createIcon(icon));
    var checkbox = document.createElement('input');
    checkbox.type = "checkbox";
    checkbox.checked = target.isVisible();
    checkbox.onclick = function() {
      target.setIsVisible(!target.isVisible());
      self.getOverlayDb().callListeners("onTargetVisibilityChange");
    };

    iconColumn.appendChild(checkbox);
  }

  var descColumn = document.createElement("td");
  result.appendChild(descColumn);

  descColumn.appendChild(self.createParamLine("Name: ", target.getName()));
  descColumn.appendChild(self.createAnnotations("Elements: ", target.getTargetParticipants(), {
    showType : false,
    inline : true
  }));
  descColumn.appendChild(self.createAnnotations("References: ", target.getReferences(), {
    showType : false,
  }));
  return result;
};

AbstractPanel.prototype.onresize = function() {
  var self = this;
  var footerPosition = window.innerHeight;
  if (self.getParent() !== undefined) {
    footerPosition = $(self.getParent().getFooter().getElement()).offset().top;
  }

  // compute the width (we can only compute it for visible elements)
  var size = 100000;

  $(".pre-scrollable", self.getElement()).each(function(index, element) {
    if ($(element).is(":visible")) {
      size = Math.min(size, footerPosition - $(element).offset().top);
    }
  });
  if (size !== 100000) {
    $(".pre-scrollable", self.getElement()).each(function(index, element) {
      $(element).css('max-height', size);
    });
  }
};

module.exports = AbstractPanel;
