"use strict";

var Promise = require("bluebird");

/* exported logger */

var AbstractDbPanel = require('./AbstractDbPanel');
var Alias = require('../../map/data/Alias');
var PanelControlElementType = require('../PanelControlElementType');
var Reaction = require('../../map/data/Reaction');

var logger = require('../../logger');
var Functions = require('../../Functions');

function SearchPanel(params) {
  params.panelName = "search";
  AbstractDbPanel.call(this, params);

  this.createSearchGui();
}

SearchPanel.prototype = Object.create(AbstractDbPanel.prototype);
SearchPanel.prototype.constructor = SearchPanel;

SearchPanel.prototype.createSearchGui = function() {
  var searchDiv = this.getControlElement(PanelControlElementType.SEARCH_DIV);

  var perfectMatchCheckbox = Functions.createElement({
    type : "input",
    name : "searchPerfectMatch",
    inputType : "checkbox",
  });
  searchDiv.appendChild(perfectMatchCheckbox);
  this.setControlElement(PanelControlElementType.SEARCH_PERFECT_MATCH_CHECKBOX, perfectMatchCheckbox);

  var perfectMatchLabel = Functions.createElement({
    type : "span",
    content : "PERFECT MATCH"
  });
  searchDiv.appendChild(perfectMatchLabel);
};

SearchPanel.prototype.createTableElement = function(element, icon) {
  if (element instanceof Alias) {
    return this.createAliasElement(element, icon);
  } else if (element instanceof Reaction) {
    return this.createReactionElement(element);
  } else {
    throw new Error("Unknown element type: " + element.constructor.name);
  }
};

SearchPanel.prototype.createPreamble = function() {
  return document.createElement("div");
};

SearchPanel.prototype.createReactionElement = function(reaction) {
  var self = this;
  var result = document.createElement("tr");
  var td = document.createElement("td");
  result.appendChild(td);
  var div = document.createElement("div");
  td.appendChild(div);

  div.appendChild(self.createLabel("Reaction: " + reaction.getReactionId()));

  if (reaction.getModelId() !== self.getMap().getId()) {
    div.appendChild(self.createSubMapLink(reaction));
  }
  div.appendChild(self.createNewLine());

  div.appendChild(self.createParamLine("Symbol: ", reaction.getSymbol()));
  div.appendChild(self.createParamLine("Abbreviation: ", reaction.getAbbreviation()));
  div.appendChild(self.createParamLine("Formula: ", reaction.getFormula()));
  div.appendChild(self.createParamLine("Mechanical Confidence Score: ", reaction.getMechanicalConfidenceScore()));
  div.appendChild(self.createParamLine("Lower Bound: ", reaction.getLowerBound()));
  div.appendChild(self.createParamLine("Upper Bound: ", reaction.getUpperBound()));
  div.appendChild(self.createParamLine("Gene Protein Reaction: ", reaction.getGeneProteinReaction()));
  div.appendChild(self.createParamLine("Subsystem: ", reaction.getSubsystem()));
  div.appendChild(self.createArrayParamLine("Synonyms: ", reaction.getSynonyms()));
  div.appendChild(self.createParamLine("Description: ", reaction.getDescription()));
  div.appendChild(self.createReactantsLine(reaction.getReactants()));
  div.appendChild(self.createProductsLine(reaction.getProducts()));
  div.appendChild(self.createModifiersLine(reaction.getModifiers()));
  div.appendChild(self.createCandidates("Candidates: ", reaction.getOther('dataMining')));
  div.appendChild(self.createAnnotations("Annotations: ", reaction.getReferences()));

  div.appendChild(self.createSeparator());
  return result;
};

SearchPanel.prototype.createReactantsLine = function(label, value) {
  var result = document.createElement("div");
  if (value !== undefined && value.length > 0) {
    for (var i = 0; i < value.length; i++) {
      result.appendChild(this.createParamLine("Reactant: ", value[i]));
    }
  }
  return result;
};

SearchPanel.prototype.createProductsLine = function(label, value) {
  var result = document.createElement("div");
  if (value !== undefined && value.length > 0) {
    for (var i = 0; i < value.length; i++) {
      result.appendChild(this.createParamLine("Product: ", value[i]));
    }
  }
  return result;
};

SearchPanel.prototype.createModifiersLine = function(label, value) {
  var result = document.createElement("div");
  if (value !== undefined && value.length > 0) {
    for (var i = 0; i < value.length; i++) {
      result.appendChild(this.createParamLine("Modifier: ", value[i]));
    }
  }
  return result;
};

SearchPanel.prototype.createAliasElement = function(alias, icon) {
  var self = this;

  var result = document.createElement("tr");
  var td = document.createElement("td");
  result.appendChild(td);
  var div = document.createElement("div");
  td.appendChild(div);

  if (icon !== undefined) {
    div.appendChild(this.createIcon(icon));
  }

  div.appendChild(this.createParamLine(alias.getType() + ": ", alias.getName()));

  if (alias.getModelId() !== self.getMap().getId()) {
    div.appendChild(self.createSubMapLink(alias));
  }
  div.appendChild(self.createNewLine(3));

  div.appendChild(self.createParamLine("Full name: ", alias.getFullName()));
  div.appendChild(self.createParamLine("Symbol: ", alias.getSymbol()));
  div.appendChild(self.createParamLine("Abbreviation: ", alias.getAbbreviation()));
  div.appendChild(self.createParamLine("Formula: ", alias.getFormula()));
  div.appendChild(self.createArrayParamLine("Former symbols: ", alias.getFormerSymbols()));
  div.appendChild(self.createPostTranslationalModifications("Posttranslational modifications: ", alias
      .getOther('posttranslationalModifications')));
  div.appendChild(self.createParamLine("Charge: ", alias.getCharge()));
  div.appendChild(self.createArrayParamLine("Synonyms: ", alias.getSynonyms()));
  div.appendChild(self.createLabelText(alias.getDescription()));
  div.appendChild(self.createCandidates("Candidates: ", alias.getOther('dataMining')));
  div.appendChild(self.createChebiTree("Chebi ontology: ", alias.getOther('chebiTree')));
  div.appendChild(self.createAnnotations("Annotations: ", alias.getReferences()));

  div.appendChild(self.createSeparator());

  return result;
};

SearchPanel.prototype.searchByQuery = function() {
  var self = this;
  var query = this.getControlElement(PanelControlElementType.SEARCH_INPUT).value;
  var perfect = this.getControlElement(PanelControlElementType.SEARCH_PERFECT_MATCH_CHECKBOX).checked;
  return self.getOverlayDb().searchByQuery(query, perfect, true);
};

SearchPanel.prototype.getAutocomplete = function(query) {
  return this.getMap().getSearchAutocomplete(query);
};

SearchPanel.prototype.init = function() {
  var query = ServerConnector.getSessionData().getSearchQuery();
  if (query !== undefined) {
    return this.getOverlayDb().searchByEncodedQuery(query, false);
  } else {
    return Promise.resolve();
  }
};

module.exports = SearchPanel;
