"use strict";

/* exported logger */

var logger = require('../../logger');

var AbstractDbOverlay = require('./AbstractDbOverlay');

var IdentifiedElement = require('../data/IdentifiedElement');

var Promise = require("bluebird");

function UserDbOverlay(params) {
  params.iconType = "marker";
  params.iconColorStart = 1;
  // call super constructor
  AbstractDbOverlay.call(this, params);

  this._markerIdentifiedElements = {
    ALIAS : [],
    REACTION : [],
    POINT : []
  };
  this._surfaceIdentifiedElements = {
    ALIAS : [],
    REACTION : [],
    POINT : []
  };
}

UserDbOverlay.prototype = Object.create(AbstractDbOverlay.prototype);
UserDbOverlay.prototype.constructor = UserDbOverlay;

UserDbOverlay.prototype.getDetailDataByIdentifiedElement = function(element) {
  var model = this.getMap().getSubmapById(element.getModelId()).getModel();
  return model.getByIdentifiedElement(element, true);
};

UserDbOverlay.prototype.addMarker = function(params) {
  var self = this;

  var elements = [];
  if (params.length === undefined) {
    params = [ params ];
  }
  var i, element;
  for (i = 0; i < params.length; i++) {
    var singleElementParam = params[i];
    element = new IdentifiedElement(singleElementParam.element);
    if (singleElementParam.icon !== undefined) {
      element.setIcon(singleElementParam.icon);
    }
    if (self._markerIdentifiedElements[element.getType()][element.getId()] !== undefined) {
      return Promise.reject(new Error("Element already highlighted: " + element.getId()));
    }
    elements.push(element);
  }
  for (i = 0; i < elements.length; i++) {
    element = elements[i];
    self._markerIdentifiedElements[element.getType()][element.getId()] = element;
  }

  return self.getIdentifiedElements().then(function(elements) {
    return self.callListeners("onSearch", {
      fitBounds : false,
      identifiedElements : elements,
    });
  });
};

UserDbOverlay.prototype.addSurface = function(params) {
  var self = this;

  var elements = [];
  if (params.length === undefined) {
    params = [ params ];
  }
  var i, element;
  for (i = 0; i < params.length; i++) {
    var singleElementParam = params[i];
    element = new IdentifiedElement(singleElementParam.element);
    if (singleElementParam.color !== undefined) {
      element.setColor(singleElementParam.color);
    }
    if (self._surfaceIdentifiedElements[element.getType()][element.getId()] !== undefined) {
      return Promise.reject(new Error("Element already highlighted: " + element.getId()));
    }
    elements.push(element);
  }
  for (i = 0; i < elements.length; i++) {
    element = elements[i];
    self._surfaceIdentifiedElements[element.getType()][element.getId()] = element;
  }

  return self.getIdentifiedElements().then(function(elements) {
    return self.callListeners("onSearch", {
      fitBounds : false,
      identifiedElements : elements,
    });
  });
};

UserDbOverlay.prototype.removeMarker = function(params) {
  var self = this;

  var elements = [];
  if (params.length === undefined) {
    params = [ params ];
  }
  if (params.length > 0) {
    var i, element;
    for (i = 0; i < params.length; i++) {
      var singleElementParam = params[i];
      element = new IdentifiedElement(singleElementParam.element);
      if (self._markerIdentifiedElements[element.getType()][element.getId()] === undefined) {
        return Promise.reject(new Error("Element is not highlighted: " + element.getId()));
      }
      elements.push(element);
    }
    for (i = 0; i < elements.length; i++) {
      element = elements[i];
      self._markerIdentifiedElements[element.getType()][element.getId()] = undefined;
      delete self._markerIdentifiedElements[element.getType()][element.getId()];
    }

    return self.getIdentifiedElements().then(function(elements) {
      return self.callListeners("onSearch", {
        fitBounds : false,
        identifiedElements : elements,
      });
    });
  } else {
    return Promise.resolve();
  }
};

UserDbOverlay.prototype.removeSurface = function(params) {
  var self = this;

  var elements = [];
  if (params.length === undefined) {
    params = [ params ];
  }
  if (params.length > 0) {
    var i, element;
    for (i = 0; i < params.length; i++) {
      var singleElementParam = params[i];
      element = new IdentifiedElement(singleElementParam.element);
      if (self._surfaceIdentifiedElements[element.getType()][element.getId()] === undefined) {
        return Promise.reject(new Error("Element is not highlighted: " + element.getId()));
      }
      elements.push(element);
    }
    for (i = 0; i < elements.length; i++) {
      element = elements[i];
      self._surfaceIdentifiedElements[element.getType()][element.getId()] = undefined;
      delete self._surfaceIdentifiedElements[element.getType()][element.getId()];
    }

    return self.getIdentifiedElements().then(function(elements) {
      return self.callListeners("onSearch", {
        fitBounds : false,
        identifiedElements : elements,
      });
    });
  } else {
    return Promise.resolve();
  }
};

UserDbOverlay.prototype.getIdentifiedElements = function() {
  var result = [];
  var markerType, key, markers;
  for (markerType in this._markerIdentifiedElements) {
    if (this._markerIdentifiedElements.hasOwnProperty(markerType)) {
      markers = this._markerIdentifiedElements[markerType];
      for (key in markers) {
        if (markers.hasOwnProperty(key)) {
          var identifiedElement = markers[key];
          if (identifiedElement.getIcon() === undefined) {
            identifiedElement.setIcon("marker/generic/search_red.png");
          }
          result.push(identifiedElement);
        }
      }
    }
  }

  for (markerType in this._surfaceIdentifiedElements) {
    if (this._surfaceIdentifiedElements.hasOwnProperty(markerType)) {
      markers = this._surfaceIdentifiedElements[markerType];
      for (key in markers) {
        if (markers.hasOwnProperty(key)) {
          result.push(markers[key]);
        }
      }
    }
  }

  return Promise.resolve(result);
};

UserDbOverlay.prototype.clear = function() {
  var self = this;
  self._markerIdentifiedElements = {
    ALIAS : [],
    REACTION : [],
    POINT : []
  };
  return self.callListeners("onSearch", {
    fitBounds : false,
    identifiedElements : [],
  });
};

module.exports = UserDbOverlay;
