"use strict";

var logger = require('../../logger');
var functions = require('../../Functions');

var AbstractInfoWindow = require('./AbstractInfoWindow');
var AliasMarker = require('../marker/AliasMarker');
var Alias = require('../data/Alias');
var LayoutAlias = require('../data/LayoutAlias');
var pileup = require('pileup');

var Promise = require("bluebird");

/**
 * Class representing info window that should be opened when clicking on alias.
 */
function AliasInfoWindow(aliasParam, map) {
  // call super constructor
  AbstractInfoWindow.call(this, aliasParam, map);

  if (aliasParam instanceof AliasMarker) {
    this.setAlias(aliasParam.getAliasData());
    map = aliasParam.getCustomMap();
  } else {
    this.setAlias(aliasParam);
  }

  var alias = this.getAlias();

  var point = new google.maps.Point(alias.x + alias.width / 2, alias.y + alias.height / 2);
  var latLng = this.customMap.fromPointToLatLng(point);
  this.googleInfowindow = new google.maps.InfoWindow({
    content : this.content,
    position : latLng
  });
}

AliasInfoWindow.prototype = Object.create(AbstractInfoWindow.prototype);
AliasInfoWindow.prototype.constructor = AliasInfoWindow;

/**
 * Updates content of info window. The content will be automatically obtained
 * from {@link CustomMap} instance. The only optional parameter is {@link Alias}
 * data.
 * 
 * @param newAlias
 *          optional {@link Alias} data for which this {@link AliasInfoWindow}
 *          was created
 */
AliasInfoWindow.prototype.update = function(newAlias) {
  if (this.alias === undefined && newAlias !== undefined) {
    this.alias = newAlias;
  }
  return this._updateContent();
};

AliasInfoWindow.prototype.init = function() {
  var self = this;
  var alias = self.getAlias();
  self.open();
  return self.getCustomMap().getTopMap().getSubmapById(alias.getModelId()).getModel().getAliasById(alias.getId(), true)
      .then(function(alias) {
        return self.update(alias);
      });
};

/**
 * Creates and returns chart representing data related to alias on different
 * layouts.
 * 
 * @returns {String} html string representing chart with data related to alias
 *          on different layouts
 */
AliasInfoWindow.prototype.createChartDiv = function() {
  var result = document.createElement("div");
  var rows = [];
  var self = this;
  return Promise.each(this.layoutAliases, function(data, i) {
    var rowDiv = document.createElement("div");
    if (i % 2 === 0) {
      rowDiv.className = "mapChartRowEvenDiv";
    } else {
      rowDiv.className = "mapChartRowOddDiv";
    }
    rowDiv.style.position = "relative";
    var nameDiv = document.createElement("div");
    nameDiv.className = "mapChartNameDiv";
    nameDiv.innerHTML = self.layoutNames[i] + "&nbsp;";
    rowDiv.appendChild(nameDiv);

    rows[i] = rowDiv;
    if (data !== undefined && data !== null) {
      return functions.overlayToColor(data).then(function(color) {
        var value = parseFloat(data.value);
        var description = data.description;
        if (description === null || description === undefined || description === "") {
          description = "";
          if (!isNaN(value)) {
            description = value.toFixed(2);
          }
        }
        var leftMarginDiv = document.createElement("div");
        leftMarginDiv.innerHTML = "&nbsp;";
        leftMarginDiv.style.float = "left";
        var centerBarDiv = document.createElement("div");
        centerBarDiv.style.width = "1px";
        centerBarDiv.style.float = "left";
        centerBarDiv.style.background = "#000000";
        centerBarDiv.innerHTML = "&nbsp;";

        var rightBarDiv = document.createElement("div");
        rightBarDiv.innerHTML = "&nbsp;";
        rightBarDiv.style.float = "left";
        rightBarDiv.style.background = color;
        rightBarDiv.style.width = Math.abs(value * 100) + "px";
        var offset = 100;
        var descDiv = document.createElement("div");
        descDiv.style.float = "right";
        descDiv.style.textAlign = "right";
        descDiv.style.position = "absolute";
        descDiv.style.right = "0";
        descDiv.innerHTML = "<span>" + description + "</span>";
        if (!isNaN(value)) {
          if (value > 0) {
            offset = 100;
            leftMarginDiv.style.width = offset + "px";

            rightBarDiv.style.textAlign = "right";

            rowDiv.appendChild(leftMarginDiv);
            rowDiv.appendChild(centerBarDiv);
            rowDiv.appendChild(rightBarDiv);
          } else {
            offset = 100 + (value * 100);
            leftMarginDiv.style.width = offset + "px";

            rowDiv.appendChild(leftMarginDiv);
            rowDiv.appendChild(rightBarDiv);
            rowDiv.appendChild(centerBarDiv);
          }

        } else {
          offset = 100;
          leftMarginDiv.style.width = offset + "px";
          leftMarginDiv.style.background = color;
          rightBarDiv.style.width = offset + "px";
          rightBarDiv.style.background = color;
          rightBarDiv.style.textAlign = "right";
          rowDiv.appendChild(leftMarginDiv);
          rowDiv.appendChild(centerBarDiv);
          rowDiv.appendChild(rightBarDiv);
        }
        rowDiv.appendChild(descDiv);
      });
    } else {
      var emptyDiv = document.createElement("div");
      emptyDiv.innerHTML = "&nbsp;";
      emptyDiv.style.float = "left";
      emptyDiv.style.width = "201px";
      rowDiv.appendChild(emptyDiv);
      return Promise.resolve();
    }
  }).then(function() {
    for (var i = 0; i < rows.length; i++) {
      result.appendChild(rows[i]);
    }
    return result;
  });
};

/**
 * Methods that creates and return DOM object with the content of the window.
 * 
 * @returns DOM object representing html code for content of the info window
 */
AliasInfoWindow.prototype.createContentDiv = function() {
  var self = this;
  var alias = self.getAlias();
  if (alias.isComplete()) {
    var result = document.createElement("div");
    var title = document.createElement("h3");
    title.innerHTML = alias.type + ": " + alias.name;
    result.appendChild(title);

    var layoutsDiv = document.createElement("div");

    result.appendChild(layoutsDiv);

    return self.getCustomMap().getAliasVisibleLayoutsData(alias.getId()).then(function(layoutAliases) {
      self.layoutAliases = layoutAliases;
      return self.getCustomMap().getTopMap().getVisibleDataOverlays();
    }).then(function(dataOverlays) {
      self.layoutNames = [];
      for (var i = 0; i < dataOverlays.length; i++) {
        logger.debug(dataOverlays[i]);
        self.layoutNames.push(dataOverlays[i].getName());
      }
      return self.createChartDiv();
    }).then(function(chartDiv) {
      layoutsDiv.appendChild(chartDiv);
      return self.createGenomicDiv();
    }).then(function(genomicDiv) {
      layoutsDiv.appendChild(genomicDiv);
      return result;
    });
  } else {
    return Promise.resolve(self.createWaitingContentDiv());
  }
};

/**
 * Returns array with data taken from all known {@link OverlayCollection}.
 * 
 * @returns array with data from {@link OverlayCollection}
 */
AliasInfoWindow.prototype.getOverlaysData = function(general) {
  return this.getCustomMap().getTopMap().getOverlayDataForAlias(this.getAlias(), general);
};

/**
 * Method returning identifier of the object for which this window was created.
 */
AliasInfoWindow.prototype.getElementId = function() {
  if (this.alias !== undefined) {
    return this.alias.getId();
  } else {
    return null;
  }
};

/**
 * Method returning type of the object for which this window was created.
 */
AliasInfoWindow.prototype.getElementType = function() {
  return "ALIAS";
};

AliasInfoWindow.prototype.createGenomicDiv = function() {
  var self = this;

  var result = document.createElement("div");
  result.style.width = '640px';

  var titleElement = document.createElement("h3");
  titleElement.innerHTML = "Gene variants";
  result.appendChild(titleElement);

  var contentElement = document.createElement("div");
  result.appendChild(contentElement);

  var geneticInformation = false;
  var genomes = [];
  var genomeUrls = [];

  var pileupSource = [ {
    viz : pileup.viz.scale(),
    name : 'Scale'
  }, {
    viz : pileup.viz.location(),
    name : 'Location'
  } ];
  var pileupRange = {
    contig : 'chr1',
    start : 3000000000,
    stop : 0
  };

  var globalGeneVariants = [];

  return Promise.each(
      self.layoutAliases,
      function(data) {
        if (data !== null && data.getType() === LayoutAlias.GENETIC_VARIANT) {
          geneticInformation = true;
          return Promise.each(data.getGeneVariants(), function(variant) {
            return self.getCustomMap().getReferenceGenome(variant.getReferenceGenomeType(),
                variant.getReferenceGenomeVersion()).then(
                function(genome) {
                  if (genome.getUrl() !== null && genome.getUrl() !== undefined) {
                    if (genomes[genome.getUrl()] === undefined) {
                      genomes[genome.getUrl()] = genome;
                      genomeUrls.push(genome.getUrl());
                    }
                  } else {
                    logger.warn("Genome for " + variant.getReferenceGenomeType() + ","
                        + variant.getReferenceGenomeVersion() + " not loaded");
                  }
                },
                function() {
                  logger.warn("Genome for " + variant.getReferenceGenomeType() + ","
                      + variant.getReferenceGenomeVersion() + " not loaded");

                });
          });

        }
      }).then(function() {
    for (var i = 0; i < genomeUrls.length; i++) {
      var genome = genomes[genomeUrls[i]];
      pileupSource.splice(0, 0, {
        viz : pileup.viz.genome(),
        isReference : pileupSource.length === 2,
        data : pileup.formats.twoBit({
          url : genome.getUrl()
        }),
        name : 'Reference ' + genome.getVersion()
      });
      for (var k = 0; k < genome.getGeneMappings().length; k++) {
        var mapping = genome.getGeneMappings()[k];
        pileupSource.push({
          viz : pileup.viz.genes(),
          data : pileup.formats.bigBed({
            url : mapping.getUrl()
          }),
          name : 'Genes ' + mapping.getName()
        });
      }
    }
    return Promise.each(self.layoutAliases, function(data, i) {
      globalGeneVariants[i] = [];
      if (data !== null && data.getType() === LayoutAlias.GENETIC_VARIANT) {
        var geneVariants = data.getGeneVariants();
        for (var j = 0; j < geneVariants.length; j++) {
          var variant = geneVariants[j];
          globalGeneVariants[i].push(variant);
          if (variant.getContig() !== undefined) {
            pileupRange.contig = variant.getContig();
          }
          pileupRange.start = Math.min(pileupRange.start, variant.getPosition());
          var length = Math.max(variant.getModifiedDna().length, variant.getOriginalDna().length);
          pileupRange.stop = Math.max(pileupRange.stop, variant.getPosition() + length);
        }
      }
    });
  }).then(function() {
    if (geneticInformation) {
      if (genomeUrls.length === 0) {
        contentElement.innerHTML = "No reference genome data available on minerva platform";
      } else {
        for (var iter = 0; iter < self.layoutAliases.length; iter++) {
          if (globalGeneVariants[iter].length > 0) {
            var vcfContent = self.createVcfString(globalGeneVariants[iter]);
            pileupSource.push({
              viz : pileup.viz.variants(),
              data : pileup.formats.vcf({
                content : vcfContent
              }),
              name : self.layoutNames[iter] + ' - Variants',
              options : {
                variantHeightByFrequency : true
              },
            });
          }
        }

        pileupRange.stop = Math.max(pileupRange.stop, pileupRange.start + 50);

        pileupRange.start--;
        pileupRange.stop++;

        if (self.pileup !== undefined) {
          logger.debug("Destroy pileup");
          self.pileup.destroy();
          logger.debug("Pileup destroyed");
        }
        self.pileup = pileup.create(contentElement, {
          range : pileupRange,
          tracks : pileupSource
        });
      }

      return result;
    } else {
      return document.createElement("div");
    }
  });
};

AliasInfoWindow.prototype.createVcfString = function(geneVariants) {
  var result = "";
  result += "##fileformat=VCFv4.0\n";
  result += "#CHROM\tPOS\tID\tREF\tALT\tQUAL\tFILTER\tINFO\n";
  for (var i = 0; i < geneVariants.length; i++) {
    var variant = geneVariants[i];
    var additionalInfo = "";
    if (variant.getAllelFrequency() !== undefined) {
      additionalInfo = "AF=" + variant.getAllelFrequency();
    }
    var variantId = ".";
    if (variant.getVariantIdentifier() !== undefined) {
      variantId = variant.getVariantIdentifier();
    }
    result += variant.getContig() + "\t" + //
    variant.getPosition() + "\t" + //
    variantId + "\t" + //
    variant.getOriginalDna() + "\t" + //
    variant.getModifiedDna() + "\t" + //
    "100.0\t" + //
    "PASS\t" + //
    additionalInfo + "\n";
  }
  return result;
};

AliasInfoWindow.prototype.setAlias = function(alias) {
  if (alias === undefined || alias === null || (!(alias instanceof Alias))) {
    throw new Error("invalid alias");
  }
  this.alias = alias;
};

AliasInfoWindow.prototype.getAlias = function() {
  return this.alias;
};

AliasInfoWindow.prototype.destroy = function() {
  if (this.pileup !== undefined) {
    this.pileup.destroy();
  }
};

module.exports = AliasInfoWindow;
