'use strict';

var functions = require('./Functions');

var IdentifiedElement = require('./map/data/IdentifiedElement');

var AbstractDbOverlay = require('./map/overlay/AbstractDbOverlay');
var DbOverlayCollection = require('./map/overlay/DbOverlayCollection');
var ConfigurationType = require('./ConfigurationType');
var CustomMap = require('./map/CustomMap');
var CustomMapOptions = require('./map/CustomMapOptions');
var SearchDbOverlay = require('./map/overlay/SearchDbOverlay');

var LeftPanel = require('./gui/leftPanel/LeftPanel');
var TopMenu = require('./gui/topMenu/TopMenu');
var Legend = require('./gui/Legend');
var MapContextMenu = require('./gui/MapContextMenu');
var SelectionContextMenu = require('./gui/SelectionContextMenu');

var OriginalGuiConnector = require('./GuiConnector');
var OriginalServerConnector = require('./ServerConnector');

var Promise = require("bluebird");

var logger = require('./logger');

var customMap, leftPanel, topMenu, legend, mapContextMenu, selectionContextMenu;

function processUrlGetParams(params) {
  var project = params.getProject();
  var sessionData = ServerConnector.getSessionData(project);

  var modelId = project.getModel().getId();
  if (GuiConnector.getParams["submap"] !== undefined) {
    modelId = parseInt(GuiConnector.getParams["submap"]);
  }

  var model = project.getModel().getSubmodelById(modelId);

  if (GuiConnector.getParams["x"] !== undefined && GuiConnector.getParams["y"] !== undefined) {
    var point = new google.maps.Point(GuiConnector.getParams["x"], GuiConnector.getParams["y"]);
    sessionData.setCenter(model, point);
  }
  if (GuiConnector.getParams["zoom"] !== undefined) {
    sessionData.setZoomLevel(model, GuiConnector.getParams["zoom"]);
  }

  if (GuiConnector.getParams["comments"] === "on") {
    sessionData.setShowComments(true);
  }
  if (GuiConnector.getParams["search"] !== undefined) {
    var query = SearchDbOverlay.prototype.encodeQuery(AbstractDbOverlay.QueryType.SEARCH_BY_QUERY,
        GuiConnector.getParams["search"]);
    sessionData.setSearchQuery(query);
  }

}

function insertGoogleAnalyticsCode() {
  return ServerConnector.getConfigurationParam(ConfigurationType.GOOGLE_ANALYTICS_IDENTIFIER).then(
      function(identifier) {
        if (identifier === "" || identifier === undefined || identifier === null) {
          return;
        } else {
          global._gaq = global._gaq || [];
          global._gaq.push([ '_setAccount', identifier ]);
          global._gaq.push([ '_trackPageview' ]);

          (function() {
            var ga = document.createElement('script');
            ga.type = 'text/javascript';
            ga.async = true;
            ga.src = ('https:' === document.location.protocol ? 'https://ssl' : 'http://www')
                + '.google-analytics.com/ga.js';
            var s = document.getElementsByTagName('script')[0];
            s.parentNode.insertBefore(ga, s);
          })();
          return;
        }
      });
}
function createDivStructure(element) {
  var leftPanelDiv = functions.createElement({
    type : "div",
    name : "leftPanelDiv",
    className : "leftPanelClass",
  });
  element.appendChild(leftPanelDiv);
  var rightPanelDiv = functions.createElement({
    type : "div",
    style : "display: table-cell;height:100%;width:100%;",
  });
  element.appendChild(rightPanelDiv);

  var rightPanelContainerDiv = functions.createElement({
    type : "div",
    style : "height:100%;width:100%;position:relative",
  });
  rightPanelDiv.appendChild(rightPanelContainerDiv);

  var menuDiv = functions.createElement({
    type : "div",
    name : "menuDiv",
    className : "menuBelt",
  });
  rightPanelContainerDiv.appendChild(menuDiv);

  var mapDiv = functions.createElement({
    type : "div",
    name : "mapDiv",
    className : "mapClass",
  });
  rightPanelContainerDiv.appendChild(mapDiv);

  var legendDiv = functions.createElement({
    type : "div",
    name : "legendDiv",
    className : "legendBox",
    style : "display:none",
  });
  rightPanelContainerDiv.appendChild(legendDiv);

  var contextMenu = functions.createElement({
    type : "ul",
    name : "contextMenu",
  });
  element.appendChild(contextMenu);
  var selectionContextMenu = functions.createElement({
    type : "ul",
    name : "selectionContextMenu",
  });
  element.appendChild(selectionContextMenu);
}

function getFullElements(customMap, identifiedElements) {
  var result = [];
  return Promise.each(
      identifiedElements,
      function(item) {
        if (item.length === undefined) {
          return customMap.getSubmapById(item.getModelId()).getModel().getByIdentifiedElement(item, true).then(
              function(fullElement) {
                result.push(fullElement);
              });
        } else {
          return getFullElements(customMap, item).then(function(resultRow) {
            result.push(resultRow);
          });
        }
      }).then(function() {
    return result;
  });
}
function getOverlayByName(customMap, dbOverlayName) {
  var dbOverlay = customMap.getOverlayByName(dbOverlayName);
  if (dbOverlay === null) {
    var validOverlays = "";
    var overlays = customMap.getDbOverlays();
    for ( var overlay in overlays) {
      if (overlays.hasOwnProperty(overlay)) {
        validOverlays += overlay.getName() + ", ";
      }
    }
    throw new Error("Invalid DbOverlay: " + dbOverlayName + ". Valid DbOverlays: " + validOverlays);
  } else {
    return dbOverlay;
  }
}

function createMarkerElements(params, type, isDefault) {
  var markerElements = [];
  if (params.length === undefined) {
    params = [ params ];
  }
  for (var i = 0; i < params.length; i++) {
    var elementParam = params[i];
    if (elementParam.options === undefined && isDefault) {
      markerElements.push({
        element : elementParam.element,
      });
    } else if (elementParam.options.type === type) {
      markerElements.push({
        element : elementParam.element,
        icon : elementParam.options.icon,
        color : elementParam.options.color,
      });
    } else if (elementParam.options.type !== "ICON" && elementParam.options.type !== "SURFACE") {
      throw new Error("Unknown type:" + elementParam.options.type);
    }
  }
  return markerElements;
}

function createResult(customMap) {
  return {
    getVisibleDataOverlays : function() {
      return customMap.getVisibleDataOverlays();
    },
    addListener : function(param) {
      var dbOverlay = getOverlayByName(customMap, param.dbOverlayName);
      dbOverlay.addListener(param.type, function(e) {
        return getFullElements(customMap, e.arg.identifiedElements).then(function(result) {
          return param.callback(result);
        });
      });
    },
    getHighlightedBioEntities : function(dbOverlayName) {
      if (dbOverlayName === undefined) {
        dbOverlayName = "user";
      }
      var dbOverlay = getOverlayByName(customMap, dbOverlayName);
      return dbOverlay.getIdentifiedElements().then(function(identifiedElements) {
        return getFullElements(customMap, identifiedElements);
      });
    },
    getProject : function() {
      return customMap.getProject();
    },
    getBioEntityById : function(param) {
      var identifiedElement = new IdentifiedElement(param);
      var model = customMap.getModel().getSubmodelById(identifiedElement.getModelId());
      return model.getByIdentifiedElement(identifiedElement, true);
    },
    getReactionsWithElement : function(param) {
      var identifiedElement = new IdentifiedElement(param);
      var model = customMap.getModel().getSubmodelById(identifiedElement.getModelId());
      return model.getReactionsForElement(identifiedElement, true);
    },
    destroy : function() {
      return leftPanel.destroy().then(function() {
        customMap.destroy();
      });
    },
    showBioEntity : function(params) {
      var iconElements = createMarkerElements(params, "ICON", true);
      var surfaceElements = createMarkerElements(params, "SURFACE", false);
      return customMap.getOverlayByName("user").addMarker(iconElements).then(function() {
        return customMap.getOverlayByName("user").addSurface(surfaceElements);
      });
    },
    hideBioEntity : function(params) {
      var iconElements = createMarkerElements(params, "ICON", true);
      var surfaceElements = createMarkerElements(params, "SURFACE", false);
      return customMap.getOverlayByName("user").removeMarker(iconElements).then(function() {
        return customMap.getOverlayByName("user").removeSurface(surfaceElements);
      });
    },
    setCenter : function(params) {
      var submap = customMap.getSubmapById(params.modelId);
      if (submap === null) {
        throw new Error("Unknown modelId: " + params.modelId);
      }
      return submap.setCenter(new google.maps.Point(params.x, params.y));
    },
    setZoom : function(params) {
      var submap = customMap.getSubmapById(params.modelId);
      if (submap === null) {
        throw new Error("Unknown modelId: " + params.modelId);
      }
      return submap.setZoom(params.zoom);
    },
  };
}

function initGlobals(params) {
  if (global.GuiConnector === undefined) {
    global.GuiConnector = OriginalGuiConnector;
    global.ServerConnector = OriginalServerConnector;
    if (params.isDebug()) {
      logger.setLevel("debug");
    } else {
      logger.setLevel("info");
    }
    GuiConnector.init();
    if (GuiConnector.getParams['debug'] !== undefined) {
      logger.setLevel("debug");
    }
  } else {
    logger.warn("global GuiConnector found");
  }

}

function verifyBrowser() {

  var browser = functions.browser;
  if (browser.name === "IE") {
    if (browser.version <= 8 || browser.compatibilityMode) {
      var message = "This webpage works only with Internet Explorer version 9 or greater.\n"
          + "If you have Internet Explorer version 9 or greater and still see this message, please, turn the 'Compatibility modeoff:\n"
          + "Open Internet Explorer and press the Alt key on your keyboard.\n"
          + "Select 'Tools' menu item. \n"
          + "Select the 'Compatibility View' settings option. \n"
          + "Make sure the 'Display all websites in Compatibility View' check box is unchecked and that the 'Compatibility View; list of websites is cleared.\n"
          + "\n" + "Alternatively, please, use other browsers: Chrome, Firefox or Safari.";
      GuiConnector.alert(message);
    }
  }
}

function getProject(params) {
  if (params.getProject() !== undefined) {
    return Promise.resolve(params.getProject());
  } else {
    return ServerConnector.getProject(params.getProjectId());
  }
}

function modifyParamsForTouchInterface(params) {
  if (params.markerOptimization === undefined && params.bigLogo === undefined
      && params.customTouchInterface === undefined) {
    var windowsTouchInterface = ((navigator.appVersion.indexOf("Win") > -1) && ('ontouchstart' in document.documentElement));
    params.markerOptimization = !windowsTouchInterface;
    params.bigLogo = windowsTouchInterface;
    params.customTouchInterface = windowsTouchInterface;
  }
  return params;
}

function create(params) {
  params = modifyParamsForTouchInterface(params);
  if (!(params instanceof CustomMapOptions)) {
    params = new CustomMapOptions(params);
  }
  initGlobals(params);
  params.getElement().style.display = "table";
  params.getElement().innerHTML = "<div style='vertical-align:middle;display:table-cell;text-align: center'>"
      + "<img src='resources/images/icons/ajax-loader.gif'/>" + "</div>";

  return getProject(params).then(function(project) {
    params.setProject(project);
    var element = params.getElement();

    verifyBrowser();

    processUrlGetParams(params);

    params.getElement().innerHTML = "";
    createDivStructure(element);
    params.setElement(functions.getElementByName(element, "mapDiv"));

    customMap = new CustomMap(params);

    new DbOverlayCollection({
      map : customMap
    });

    leftPanel = new LeftPanel({
      element : functions.getElementByName(element, "leftPanelDiv"),
      customMap : customMap
    });

    topMenu = new TopMenu({
      element : functions.getElementByName(element, "menuDiv"),
      customMap : customMap
    });

    legend = new Legend({
      element : functions.getElementByName(element, "legendDiv"),
      customMap : customMap
    });

    mapContextMenu = new MapContextMenu({
      element : functions.getElementByName(element, "contextMenu"),
      customMap : customMap
    });
    customMap.setContextMenu(mapContextMenu);

    selectionContextMenu = new SelectionContextMenu({
      element : functions.getElementByName(element, "selectionContextMenu"),
      customMap : customMap
    });
    customMap.setSelectionContextMenu(selectionContextMenu);

    topMenu.setLegend(legend);
    topMenu.setLeftPanel(leftPanel);

    return customMap.init();
  }).then(function() {
    return insertGoogleAnalyticsCode(customMap);
  }).then(function() {
    return leftPanel.init();
  }).then(function() {
    return legend.init();
  }).then(function() {
    return topMenu.init();
  }).then(function() {
    return selectionContextMenu.init();
  }).then(function() {
    return mapContextMenu.init();
  }).then(function() {
    if (GuiConnector.getParams["layout"] !== undefined) {
      var layouts = params.getProject().getModel().getLayouts();
      for (var j = 0; j < layouts.length; j++) {
        var layout = layouts[j];
        if (layout.getName() === GuiConnector.getParams["layout"]) {
          return customMap.openDataOverlay(layout);
        }
      }
    }
  }).then(function() {

    var submapId = GuiConnector.getParams["submap"];
    if (submapId !== undefined) {
      return customMap.openSubmap(submapId);
    }
  }).then(function() {
    var result = createResult(customMap);

    if (params.isDebug()) {
      result.leftPanel = leftPanel;
      result.customMap = customMap;
    }
    return result;
  });

}

var minerva = {
  create : create,
  ServerConnector : OriginalServerConnector,
  GuiConnector : OriginalGuiConnector,
};

module.exports = minerva;
